# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import testtools

import shade
from shade.tests.unit import base


zone_dict = {
    'name': 'example.net.',
    'type': 'PRIMARY',
    'email': 'test@example.net',
    'description': 'Example zone',
    'ttl': 3600,
}

new_zone_dict = copy.copy(zone_dict)
new_zone_dict['id'] = '1'


class TestZone(base.RequestsMockTestCase):

    def setUp(self):
        super(TestZone, self).setUp()
        self.use_designate()

    def test_create_zone(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json=new_zone_dict,
                 validate=dict(
                     json=zone_dict))
        ])
        z = self.cloud.create_zone(
            name=zone_dict['name'],
            zone_type=zone_dict['type'],
            email=zone_dict['email'],
            description=zone_dict['description'],
            ttl=zone_dict['ttl'],
            masters=None)
        self.assertEqual(new_zone_dict, z)
        self.assert_calls()

    def test_create_zone_exception(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 status_code=500)
        ])
        with testtools.ExpectedException(
            shade.exc.OpenStackCloudHTTPError,
            "Unable to create zone example.net."
        ):
            self.cloud.create_zone('example.net.')
        self.assert_calls()

    def test_update_zone(self):
        new_ttl = 7200
        updated_zone = copy.copy(new_zone_dict)
        updated_zone['ttl'] = new_ttl
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [new_zone_dict],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='PATCH',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones', '1']),
                 json=updated_zone,
                 validate=dict(
                     json={"ttl": new_ttl}))
        ])
        z = self.cloud.update_zone('1', ttl=new_ttl)
        self.assertEqual(updated_zone, z)
        self.assert_calls()

    def test_delete_zone(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [new_zone_dict],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones', '1']),
                 json=new_zone_dict)
        ])
        self.assertTrue(self.cloud.delete_zone('1'))
        self.assert_calls()

    def test_get_zone_by_id(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [new_zone_dict],
                     "links": {},
                     "metadata": {
                         'total_count': 1}})
        ])
        zone = self.cloud.get_zone('1')
        self.assertEqual(zone['id'], '1')
        self.assert_calls()

    def test_get_zone_by_name(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [new_zone_dict],
                     "links": {},
                     "metadata": {
                         'total_count': 1}})
        ])
        zone = self.cloud.get_zone('example.net.')
        self.assertEqual(zone['name'], 'example.net.')
        self.assert_calls()

    def test_get_zone_not_found_returns_false(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [],
                     "links": {},
                     "metadata": {
                         'total_count': 1}})
        ])
        zone = self.cloud.get_zone('nonexistingzone.net.')
        self.assertFalse(zone)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from shade.tests.unit import base
from shade.tests import fakes


class TestAggregate(base.RequestsMockTestCase):

    def setUp(self):
        super(TestAggregate, self).setUp()
        self.aggregate_name = self.getUniqueString('aggregate')
        self.fake_aggregate = fakes.make_fake_aggregate(1, self.aggregate_name)

    def test_create_aggregate(self):
        create_aggregate = self.fake_aggregate.copy()
        del create_aggregate['metadata']
        del create_aggregate['hosts']

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregate': create_aggregate},
                 validate=dict(json={
                     'aggregate': {
                         'name': self.aggregate_name,
                         'availability_zone': None,
                     }})),
        ])
        self.op_cloud.create_aggregate(name=self.aggregate_name)

        self.assert_calls()

    def test_create_aggregate_with_az(self):
        availability_zone = 'az1'
        az_aggregate = fakes.make_fake_aggregate(
            1, self.aggregate_name, availability_zone=availability_zone)

        create_aggregate = az_aggregate.copy()
        del create_aggregate['metadata']
        del create_aggregate['hosts']

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregate': create_aggregate},
                 validate=dict(json={
                     'aggregate': {
                         'name': self.aggregate_name,
                         'availability_zone': availability_zone,
                     }})),
        ])

        self.op_cloud.create_aggregate(
            name=self.aggregate_name, availability_zone=availability_zone)

        self.assert_calls()

    def test_delete_aggregate(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates', '1'])),
        ])

        self.assertTrue(self.op_cloud.delete_aggregate('1'))

        self.assert_calls()

    def test_update_aggregate_set_az(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates', '1']),
                 json={'aggregate': self.fake_aggregate},
                 validate=dict(
                     json={
                         'aggregate': {
                             'availability_zone': 'az',
                         }})),
        ])

        self.op_cloud.update_aggregate(1, availability_zone='az')

        self.assert_calls()

    def test_update_aggregate_unset_az(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates', '1']),
                 json={'aggregate': self.fake_aggregate},
                 validate=dict(
                     json={
                         'aggregate': {
                             'availability_zone': None,
                         }})),
        ])

        self.op_cloud.update_aggregate(1, availability_zone=None)

        self.assert_calls()

    def test_set_aggregate_metadata(self):
        metadata = {'key': 'value'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-aggregates', '1', 'action']),
                 json={'aggregate': self.fake_aggregate},
                 validate=dict(
                     json={'set_metadata': {'metadata': metadata}})),
        ])
        self.op_cloud.set_aggregate_metadata('1', metadata)

        self.assert_calls()

    def test_add_host_to_aggregate(self):
        hostname = 'host1'
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-aggregates', '1', 'action']),
                 json={'aggregate': self.fake_aggregate},
                 validate=dict(
                     json={'add_host': {'host': hostname}})),
        ])
        self.op_cloud.add_host_to_aggregate('1', hostname)

        self.assert_calls()

    def test_remove_host_from_aggregate(self):
        hostname = 'host1'
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-aggregates']),
                 json={'aggregates': [self.fake_aggregate]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-aggregates', '1', 'action']),
                 json={'aggregate': self.fake_aggregate},
                 validate=dict(
                     json={'remove_host': {'host': hostname}})),
        ])
        self.op_cloud.remove_host_from_aggregate('1', hostname)

        self.assert_calls()



import shade
shade.simple_logging(debug=True)

cloud = shade.openstack_cloud(cloud='my-citycloud', region_name='Buf1')
try:
    server = cloud.create_server(
        'my-server', image='Ubuntu 16.04 Xenial Xerus',
        flavor=dict(id='0dab10b5-42a2-438e-be7b-505741a7ffcc'),
        wait=True, auto_ip=True)

    print("\n\nFull Server\n\n")
    cloud.pprint(server)

    print("\n\nTurn Detailed Off\n\n")
    cloud.pprint(cloud.get_server('my-server', detailed=False))

    print("\n\nBare Server\n\n")
    cloud.pprint(cloud.get_server('my-server', bare=True))

finally:
    # Delete it - this is a demo
    cloud.delete_server(server, wait=True, delete_ips=True)




# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import copy
import testtools

import shade
from shade.tests.unit import base


zone = {
    'id': '1',
    'name': 'example.net.',
    'type': 'PRIMARY',
    'email': 'test@example.net',
    'description': 'Example zone',
    'ttl': 3600,
}

recordset = {
    'name': 'www.example.net.',
    'type': 'A',
    'description': 'Example zone',
    'ttl': 3600,
    'records': ['192.168.1.1']
}
recordset_zone = '1'

new_recordset = copy.copy(recordset)
new_recordset['id'] = '1'
new_recordset['zone'] = recordset_zone


class TestRecordset(base.RequestsMockTestCase):

    def setUp(self):
        super(TestRecordset, self).setUp()
        self.use_designate()

    def test_create_recordset(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [zone],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'], 'recordsets']),
                 json=new_recordset,
                 validate=dict(json=recordset)),
        ])
        rs = self.cloud.create_recordset(
            zone=recordset_zone,
            name=recordset['name'],
            recordset_type=recordset['type'],
            records=recordset['records'],
            description=recordset['description'],
            ttl=recordset['ttl'])
        self.assertEqual(new_recordset, rs)
        self.assert_calls()

    def test_create_recordset_exception(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [zone],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'], 'recordsets']),
                 status_code=500,
                 validate=dict(json={
                     'name': 'www2.example.net.',
                     'records': ['192.168.1.2'],
                     'type': 'A'})),
        ])
        with testtools.ExpectedException(
            shade.exc.OpenStackCloudHTTPError,
            "Error creating recordset www2.example.net."
        ):
            self.cloud.create_recordset('1', 'www2.example.net.',
                                        'a', ['192.168.1.2'])
        self.assert_calls()

    def test_update_recordset(self):
        new_ttl = 7200
        expected_recordset = {
            'name': recordset['name'],
            'records': recordset['records'],
            'type': recordset['type']
        }
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [zone],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'],
                             'recordsets', new_recordset['id']]),
                 json=new_recordset),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'],
                             'recordsets', new_recordset['id']]),
                 json=expected_recordset,
                 validate=dict(json={'ttl': new_ttl}))
        ])
        updated_rs = self.cloud.update_recordset('1', '1', ttl=new_ttl)
        self.assertEqual(expected_recordset, updated_rs)
        self.assert_calls()

    def test_delete_recordset(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public', append=['v2', 'zones']),
                 json={
                     "zones": [zone],
                     "links": {},
                     "metadata": {
                         'total_count': 1}}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'],
                             'recordsets', new_recordset['id']]),
                 json=new_recordset),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', zone['id'],
                             'recordsets', new_recordset['id']]),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_recordset('1', '1'))
        self.assert_calls()

    def test_get_recordset_by_id(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', '1', 'recordsets', '1']),
                 json=new_recordset),
        ])
        recordset = self.cloud.get_recordset('1', '1')
        self.assertEqual(recordset['id'], '1')
        self.assert_calls()

    def test_get_recordset_by_name(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', '1', 'recordsets',
                             new_recordset['name']]),
                 json=new_recordset),
        ])
        recordset = self.cloud.get_recordset('1', new_recordset['name'])
        self.assertEqual(new_recordset['name'], recordset['name'])
        self.assert_calls()

    def test_get_recordset_not_found_returns_false(self):
        recordset_name = "www.nonexistingrecord.net."
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'dns', 'public',
                     append=['v2', 'zones', '1', 'recordsets',
                             recordset_name]),
                 json=[])
        ])
        recordset = self.cloud.get_recordset('1', recordset_name)
        self.assertFalse(recordset)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock
import uuid

import testtools

import shade
from shade import _utils
from shade import exc
from shade.tests import fakes
from shade.tests.unit import base


RANGE_DATA = [
    dict(id=1, key1=1, key2=5),
    dict(id=2, key1=1, key2=20),
    dict(id=3, key1=2, key2=10),
    dict(id=4, key1=2, key2=30),
    dict(id=5, key1=3, key2=40),
    dict(id=6, key1=3, key2=40),
]


class TestShade(base.RequestsMockTestCase):

    def setUp(self):
        # This set of tests are not testing neutron, they're testing
        # rebuilding servers, but we do several network calls in service
        # of a NORMAL rebuild to find the default_network. Putting
        # in all of the neutron mocks for that will make the tests harder
        # to read. SO - we're going mock neutron into the off position
        # and then turn it back on in the few tests that specifically do.
        # Maybe we should reorg these into two classes - one with neutron
        # mocked out - and one with it not mocked out
        super(TestShade, self).setUp()
        self.has_neutron = False

        def fake_has_service(*args, **kwargs):
            return self.has_neutron

        self.cloud.has_service = fake_has_service

    def test_openstack_cloud(self):
        self.assertIsInstance(self.cloud, shade.OpenStackCloud)

    @mock.patch.object(shade.OpenStackCloud, 'search_images')
    def test_get_images(self, mock_search):
        image1 = dict(id='123', name='mickey')
        mock_search.return_value = [image1]
        r = self.cloud.get_image('mickey')
        self.assertIsNotNone(r)
        self.assertDictEqual(image1, r)

    @mock.patch.object(shade.OpenStackCloud, 'search_images')
    def test_get_image_not_found(self, mock_search):
        mock_search.return_value = []
        r = self.cloud.get_image('doesNotExist')
        self.assertIsNone(r)

    def test_get_server(self):
        server1 = fakes.make_fake_server('123', 'mickey')
        server2 = fakes.make_fake_server('345', 'mouse')

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server1, server2]}),
        ])

        r = self.cloud.get_server('mickey')
        self.assertIsNotNone(r)
        self.assertEqual(server1['name'], r['name'])

        self.assert_calls()

    def test_get_server_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': []}),
        ])

        r = self.cloud.get_server('doesNotExist')
        self.assertIsNone(r)

        self.assert_calls()

    def test_list_servers_exception(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 status_code=400)
        ])

        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_servers)

        self.assert_calls()

    def test__neutron_exceptions_resource_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 status_code=404)
        ])
        self.assertRaises(exc.OpenStackCloudResourceNotFound,
                          self.cloud.list_networks)
        self.assert_calls()

    def test__neutron_exceptions_url_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 status_code=404)
        ])
        self.assertRaises(exc.OpenStackCloudURINotFound,
                          self.cloud.list_networks)
        self.assert_calls()

    def test_list_servers(self):
        server_id = str(uuid.uuid4())
        server_name = self.getUniqueString('name')
        fake_server = fakes.make_fake_server(server_id, server_name)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [fake_server]}),
        ])

        r = self.cloud.list_servers()

        self.assertEqual(1, len(r))
        self.assertEqual(server_name, r[0]['name'])

        self.assert_calls()

    def test_list_servers_all_projects(self):
        '''This test verifies that when list_servers is called with
        `all_projects=True` that it passes `all_tenants=True` to nova.'''
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail'],
                     qs_elements=['all_tenants=True']),
                 complete_qs=True,
                 json={'servers': []}),
        ])

        self.cloud.list_servers(all_projects=True)

        self.assert_calls()

    def test_iterate_timeout_bad_wait(self):
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Wait value must be an int or float value."):
            for count in _utils._iterate_timeout(
                    1, "test_iterate_timeout_bad_wait", wait="timeishard"):
                pass

    @mock.patch('time.sleep')
    def test_iterate_timeout_str_wait(self, mock_sleep):
        iter = _utils._iterate_timeout(
            10, "test_iterate_timeout_str_wait", wait="1.6")
        next(iter)
        next(iter)
        mock_sleep.assert_called_with(1.6)

    @mock.patch('time.sleep')
    def test_iterate_timeout_int_wait(self, mock_sleep):
        iter = _utils._iterate_timeout(
            10, "test_iterate_timeout_int_wait", wait=1)
        next(iter)
        next(iter)
        mock_sleep.assert_called_with(1.0)

    @mock.patch('time.sleep')
    def test_iterate_timeout_timeout(self, mock_sleep):
        message = "timeout test"
        with testtools.ExpectedException(
                exc.OpenStackCloudTimeout,
                message):
            for count in _utils._iterate_timeout(0.1, message, wait=1):
                pass
        mock_sleep.assert_called_with(1.0)

    def test__nova_extensions(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support."
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension."
            },
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=dict(extensions=body))
        ])
        extensions = self.cloud._nova_extensions()
        self.assertEqual(set(['NMN', 'OS-DCF']), extensions)

        self.assert_calls()

    def test__nova_extensions_fails(self):
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 status_code=404),
        ])
        with testtools.ExpectedException(
            exc.OpenStackCloudURINotFound,
            "Error fetching extension list for nova"
        ):
            self.cloud._nova_extensions()

        self.assert_calls()

    def test__has_nova_extension(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support."
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension."
            },
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=dict(extensions=body))
        ])
        self.assertTrue(self.cloud._has_nova_extension('NMN'))

        self.assert_calls()

    def test__has_nova_extension_missing(self):
        body = [
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "Multinic",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "NMN",
                "description": "Multiple network support."
            },
            {
                "updated": "2014-12-03T00:00:00Z",
                "name": "DiskConfig",
                "links": [],
                "namespace": "http://openstack.org/compute/ext/fake_xml",
                "alias": "OS-DCF",
                "description": "Disk Management Extension."
            },
        ]
        self.register_uris([
            dict(method='GET',
                 uri='{endpoint}/extensions'.format(
                     endpoint=fakes.COMPUTE_ENDPOINT),
                 json=dict(extensions=body))
        ])
        self.assertFalse(self.cloud._has_nova_extension('invalid'))

        self.assert_calls()

    def test_range_search(self):
        filters = {"key1": "min", "key2": "20"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(1, len(retval))
        self.assertEqual([RANGE_DATA[1]], retval)

    def test_range_search_2(self):
        filters = {"key1": "<=2", "key2": ">10"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(2, len(retval))
        self.assertEqual([RANGE_DATA[1], RANGE_DATA[3]], retval)

    def test_range_search_3(self):
        filters = {"key1": "2", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(0, len(retval))

    def test_range_search_4(self):
        filters = {"key1": "max", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(0, len(retval))

    def test_range_search_5(self):
        filters = {"key1": "min", "key2": "min"}
        retval = self.cloud.range_search(RANGE_DATA, filters)
        self.assertIsInstance(retval, list)
        self.assertEqual(1, len(retval))
        self.assertEqual([RANGE_DATA[0]], retval)



# Copyright 2017 OVH SAS
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import testtools

from shade import exc
from shade.tests.unit import base


class TestSubnet(base.RequestsMockTestCase):

    network_name = 'network_name'
    subnet_name = 'subnet_name'
    subnet_id = '1f1696eb-7f47-47f6-835c-4889bff88604'
    subnet_cidr = '192.168.199.0/24'

    mock_network_rep = {
        'id': '881d1bb7-a663-44c0-8f9f-ee2765b74486',
        'name': network_name,
    }

    mock_subnet_rep = {
        'allocation_pools': [{
            'start': u'192.168.199.2',
            'end': u'192.168.199.254'
        }],
        'cidr': subnet_cidr,
        'created_at': '2017-04-24T20:22:23Z',
        'description': '',
        'dns_nameservers': [],
        'enable_dhcp': False,
        'gateway_ip': '192.168.199.1',
        'host_routes': [],
        'id': subnet_id,
        'ip_version': 4,
        'ipv6_address_mode': None,
        'ipv6_ra_mode': None,
        'name': subnet_name,
        'network_id': mock_network_rep['id'],
        'project_id': '861808a93da0484ea1767967c4df8a23',
        'revision_number': 2,
        'service_types': [],
        'subnetpool_id': None,
        'tags': []
    }

    def test_get_subnet(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [self.mock_subnet_rep]})
        ])
        r = self.cloud.get_subnet(self.subnet_name)
        self.assertIsNotNone(r)
        self.assertDictEqual(self.mock_subnet_rep, r)
        self.assert_calls()

    def test_create_subnet(self):
        pool = [{'start': '192.168.199.2', 'end': '192.168.199.254'}]
        dns = ['8.8.8.8']
        routes = [{"destination": "0.0.0.0/0", "nexthop": "123.456.78.9"}]
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['host_routes'] = routes
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnet': mock_subnet_rep},
                 validate=dict(
                     json={'subnet': {
                         'cidr': self.subnet_cidr,
                         'enable_dhcp': False,
                         'ip_version': 4,
                         'network_id': self.mock_network_rep['id'],
                         'allocation_pools': pool,
                         'dns_nameservers': dns,
                         'host_routes': routes}}))
        ])
        subnet = self.cloud.create_subnet(self.network_name, self.subnet_cidr,
                                          allocation_pools=pool,
                                          dns_nameservers=dns,
                                          host_routes=routes)
        self.assertDictEqual(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_string_ip_version(self):
        '''Allow ip_version as a string'''
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnet': self.mock_subnet_rep},
                 validate=dict(
                     json={'subnet': {
                         'cidr': self.subnet_cidr,
                         'enable_dhcp': False,
                         'ip_version': 4,
                         'network_id': self.mock_network_rep['id']}}))
        ])
        subnet = self.cloud.create_subnet(
            self.network_name, self.subnet_cidr, ip_version='4')
        self.assertDictEqual(self.mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_bad_ip_version(self):
        '''String ip_versions must be convertable to int'''
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]})
        ])
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "ip_version must be an integer"
        ):
            self.cloud.create_subnet(
                self.network_name, self.subnet_cidr, ip_version='4x')
        self.assert_calls()

    def test_create_subnet_without_gateway_ip(self):
        pool = [{'start': '192.168.199.2', 'end': '192.168.199.254'}]
        dns = ['8.8.8.8']
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['gateway_ip'] = None
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnet': mock_subnet_rep},
                 validate=dict(
                     json={'subnet': {
                         'cidr': self.subnet_cidr,
                         'enable_dhcp': False,
                         'ip_version': 4,
                         'network_id': self.mock_network_rep['id'],
                         'allocation_pools': pool,
                         'gateway_ip': None,
                         'dns_nameservers': dns}}))
        ])
        subnet = self.cloud.create_subnet(self.network_name, self.subnet_cidr,
                                          allocation_pools=pool,
                                          dns_nameservers=dns,
                                          disable_gateway_ip=True)
        self.assertDictEqual(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_with_gateway_ip(self):
        pool = [{'start': '192.168.199.8', 'end': '192.168.199.254'}]
        gateway = '192.168.199.2'
        dns = ['8.8.8.8']
        mock_subnet_rep = copy.copy(self.mock_subnet_rep)
        mock_subnet_rep['allocation_pools'] = pool
        mock_subnet_rep['dns_nameservers'] = dns
        mock_subnet_rep['gateway_ip'] = gateway
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnet': mock_subnet_rep},
                 validate=dict(
                     json={'subnet': {
                         'cidr': self.subnet_cidr,
                         'enable_dhcp': False,
                         'ip_version': 4,
                         'network_id': self.mock_network_rep['id'],
                         'allocation_pools': pool,
                         'gateway_ip': gateway,
                         'dns_nameservers': dns}}))
        ])
        subnet = self.cloud.create_subnet(self.network_name, self.subnet_cidr,
                                          allocation_pools=pool,
                                          dns_nameservers=dns,
                                          gateway_ip=gateway)
        self.assertDictEqual(mock_subnet_rep, subnet)
        self.assert_calls()

    def test_create_subnet_conflict_gw_ops(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]})
        ])
        gateway = '192.168.200.3'
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.create_subnet, 'kooky',
                          self.subnet_cidr, gateway_ip=gateway,
                          disable_gateway_ip=True)
        self.assert_calls()

    def test_create_subnet_bad_network(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [self.mock_network_rep]})
        ])
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.create_subnet,
                          'duck', self.subnet_cidr)
        self.assert_calls()

    def test_create_subnet_non_unique_network(self):
        net1 = dict(id='123', name=self.network_name)
        net2 = dict(id='456', name=self.network_name)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'networks.json']),
                 json={'networks': [net1, net2]})
        ])
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.create_subnet,
                          self.network_name, self.subnet_cidr)
        self.assert_calls()

    def test_delete_subnet(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [self.mock_subnet_rep]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_subnet(self.subnet_name))
        self.assert_calls()

    def test_delete_subnet_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': []})
        ])
        self.assertFalse(self.cloud.delete_subnet('goofy'))
        self.assert_calls()

    def test_delete_subnet_multiple_found(self):
        subnet1 = dict(id='123', name=self.subnet_name)
        subnet2 = dict(id='456', name=self.subnet_name)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [subnet1, subnet2]})
        ])
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.delete_subnet,
                          self.subnet_name)
        self.assert_calls()

    def test_delete_subnet_multiple_using_id(self):
        subnet1 = dict(id='123', name=self.subnet_name)
        subnet2 = dict(id='456', name=self.subnet_name)
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [subnet1, subnet2]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'subnets', '%s.json' % subnet1['id']]),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_subnet(subnet1['id']))
        self.assert_calls()

    def test_update_subnet(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        expected_subnet['name'] = 'goofy'
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [self.mock_subnet_rep]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
                 json={'subnet': expected_subnet},
                 validate=dict(
                     json={'subnet': {'name': 'goofy'}}))
        ])
        subnet = self.cloud.update_subnet(self.subnet_id, subnet_name='goofy')
        self.assertDictEqual(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_gateway_ip(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        gateway = '192.168.199.3'
        expected_subnet['gateway_ip'] = gateway
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [self.mock_subnet_rep]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
                 json={'subnet': expected_subnet},
                 validate=dict(
                     json={'subnet': {'gateway_ip': gateway}}))
        ])
        subnet = self.cloud.update_subnet(self.subnet_id, gateway_ip=gateway)
        self.assertDictEqual(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_disable_gateway_ip(self):
        expected_subnet = copy.copy(self.mock_subnet_rep)
        expected_subnet['gateway_ip'] = None
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'subnets.json']),
                 json={'subnets': [self.mock_subnet_rep]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'subnets', '%s.json' % self.subnet_id]),
                 json={'subnet': expected_subnet},
                 validate=dict(
                     json={'subnet': {'gateway_ip': None}}))
        ])
        subnet = self.cloud.update_subnet(self.subnet_id,
                                          disable_gateway_ip=True)
        self.assertDictEqual(expected_subnet, subnet)
        self.assert_calls()

    def test_update_subnet_conflict_gw_ops(self):
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.update_subnet,
                          self.subnet_id, gateway_ip="192.168.199.3",
                          disable_gateway_ip=True)



import shade

# Initialize and turn on debug logging
shade.simple_logging(debug=True)

for cloud_name, region_name in [
        ('my-vexxhost', 'ca-ymq-1'),
        ('my-citycloud', 'Buf1'),
        ('my-internap', 'ams01')]:
    # Initialize cloud
    cloud = shade.openstack_cloud(cloud=cloud_name, region_name=region_name)
    for server in cloud.search_servers('my-server'):
        cloud.delete_server(server, wait=True, delete_ips=True)



import shade
shade.simple_logging()

cloud = shade.openstack_cloud(cloud='fuga', region_name='cystack')
image = cloud.get_image(
    'Ubuntu 16.04 LTS - Xenial Xerus - 64-bit - Fuga Cloud Based Image')
cloud.pprint(image)



import shade
shade.simple_logging()

cloud = shade.openstack_cloud(cloud='fuga', region_name='cystack')
cloud.pprint([
    image for image in cloud.list_images()
    if 'ubuntu' in image.name.lower()])



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.

from shade import task_manager


class UserCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.create(**self.args)


class UserUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.update(**self.args)


class UserPasswordUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.update_password(**self.args)


class UserAddToGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.add_to_group(**self.args)


class UserCheckInGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.check_in_group(**self.args)


class UserRemoveFromGroup(task_manager.Task):
    def main(self, client):
        return client.keystone_client.users.remove_from_group(**self.args)


class MachineCreate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.create(**self.args)


class MachineDelete(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.delete(**self.args)


class MachinePatch(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.update(**self.args)


class MachinePortGet(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.get(**self.args)


class MachinePortGetByAddress(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.get_by_address(**self.args)


class MachinePortCreate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.create(**self.args)


class MachinePortDelete(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.delete(**self.args)


class MachinePortList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.port.list()


class MachineNodeGet(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.get(**self.args)


class MachineNodeList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.list(**self.args)


class MachineNodePortList(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.list_ports(**self.args)


class MachineNodeUpdate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.update(**self.args)


class MachineNodeValidate(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.validate(**self.args)


class MachineSetMaintenance(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_maintenance(**self.args)


class MachineSetPower(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_power_state(**self.args)


class MachineSetProvision(task_manager.Task):
    def main(self, client):
        return client.ironic_client.node.set_provision_state(**self.args)


class ServiceCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.create(**self.args)


class ServiceList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.list()


class ServiceUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.update(**self.args)


class ServiceDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.services.delete(**self.args)


class EndpointCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.create(**self.args)


class EndpointUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.update(**self.args)


class EndpointList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.list()


class EndpointDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.endpoints.delete(**self.args)


class DomainCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.create(**self.args)


class DomainList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.list(**self.args)


class DomainGet(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.get(**self.args)


class DomainUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.update(**self.args)


class DomainDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.domains.delete(**self.args)


class GroupList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.list()


class GroupCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.create(**self.args)


class GroupDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.delete(**self.args)


class GroupUpdate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.groups.update(**self.args)


class RoleList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.list()


class RoleCreate(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.create(**self.args)


class RoleDelete(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.delete(**self.args)


class RoleAddUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.add_user_role(**self.args)


class RoleGrantUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.grant(**self.args)


class RoleRemoveUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.remove_user_role(**self.args)


class RoleRevokeUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.revoke(**self.args)


class RoleAssignmentList(task_manager.Task):
    def main(self, client):
        return client.keystone_client.role_assignments.list(**self.args)


class RolesForUser(task_manager.Task):
    def main(self, client):
        return client.keystone_client.roles.roles_for_user(**self.args)



import shade
shade.simple_logging(debug=True)

cloud = shade.openstack_cloud(cloud='ovh', region_name='SBG1')
cloud.create_object(
    container='my-container', name='my-object',
    filename='/home/mordred/briarcliff.sh3d',
    segment_size=1000000)
cloud.delete_object('my-container', 'my-object')
cloud.delete_container('my-container')



#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from shade import exc
from shade.tests import fakes
from shade.tests.unit import base


class TestKeypair(base.RequestsMockTestCase):

    def setUp(self):
        super(TestKeypair, self).setUp()
        self.keyname = self.getUniqueString('key')
        self.key = fakes.make_fake_keypair(self.keyname)

    def test_create_keypair(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-keypairs']),
                 json={'keypair': self.key},
                 validate=dict(json={
                     'keypair': {
                         'name': self.key['name'],
                         'public_key': self.key['public_key']}})),
        ])

        new_key = self.cloud.create_keypair(
            self.keyname, self.key['public_key'])
        self.assertEqual(new_key, self.cloud._normalize_keypair(self.key))

        self.assert_calls()

    def test_create_keypair_exception(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-keypairs']),
                 status_code=400,
                 validate=dict(json={
                     'keypair': {
                         'name': self.key['name'],
                         'public_key': self.key['public_key']}})),
        ])

        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.create_keypair,
            self.keyname, self.key['public_key'])

        self.assert_calls()

    def test_delete_keypair(self):
        self.register_uris([
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-keypairs', self.keyname]),
                 status_code=202),
        ])
        self.assertTrue(self.cloud.delete_keypair(self.keyname))

        self.assert_calls()

    def test_delete_keypair_not_found(self):
        self.register_uris([
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-keypairs', self.keyname]),
                 status_code=404),
        ])
        self.assertFalse(self.cloud.delete_keypair(self.keyname))

        self.assert_calls()

    def test_list_keypairs(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-keypairs']),
                 json={'keypairs': [{'keypair': self.key}]}),

        ])
        keypairs = self.cloud.list_keypairs()
        self.assertEqual(keypairs, self.cloud._normalize_keypairs([self.key]))
        self.assert_calls()

    def test_list_keypairs_exception(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-keypairs']),
                 status_code=400),

        ])
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.list_keypairs)
        self.assert_calls()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import importlib
import warnings

from os_client_config import constructors

from shade import _utils


class LegacyClientFactoryMixin(object):
    """Mixin Class containing factory functions for legacy client objects.

    Methods in this class exist for backwards compatibility so will not go
    away any time - but they are all things whose use is discouraged. They're
    in a mixin to unclutter the main class file.
    """

    def _create_legacy_client(
            self, client, service, deprecated=True,
            module_name=None, **kwargs):
        if client not in self._legacy_clients:
            if deprecated:
                self._deprecated_import_check(client)
            if module_name:
                constructors.get_constructor_mapping()[service] = module_name
            self._legacy_clients[client] = self._get_client(service, **kwargs)
        return self._legacy_clients[client]

    def _deprecated_import_check(self, client):
        module_name = '{client}client'.format(client=client)
        warnings.warn(
            'Using shade to get a {module_name} object is deprecated. If you'
            ' need a {module_name} object, please use make_legacy_client in'
            ' os-client-config instead'.format(module_name=module_name))
        try:
            importlib.import_module(module_name)
        except ImportError:
            self.log.error(
                '{module_name} is no longer a dependency of shade. You need to'
                ' install python-{module_name} directly.'.format(
                    module_name=module_name))
            raise

    @property
    def trove_client(self):
        return self._create_legacy_client('trove', 'database')

    @property
    def magnum_client(self):
        return self._create_legacy_client('magnum', 'container-infra')

    @property
    def neutron_client(self):
        return self._create_legacy_client('neutron', 'network')

    @property
    def nova_client(self):
        return self._create_legacy_client('nova', 'compute', version='2.0')

    @property
    def glance_client(self):
        return self._create_legacy_client('glance', 'image')

    @property
    def heat_client(self):
        return self._create_legacy_client('heat', 'orchestration')

    @property
    def swift_client(self):
        return self._create_legacy_client('swift', 'object-store')

    @property
    def cinder_client(self):
        return self._create_legacy_client('cinder', 'volume')

    @property
    def designate_client(self):
        return self._create_legacy_client('designate', 'dns')

    @property
    def keystone_client(self):
        # Trigger discovery from ksa
        self._identity_client

        # Skip broken discovery in ksc. We're good thanks.
        from keystoneclient.v2_0 import client as v2_client
        from keystoneclient.v3 import client as v3_client
        if self.cloud_config.config['identity_api_version'] == '3':
            client_class = v3_client
        else:
            client_class = v2_client

        return self._create_legacy_client(
            'keystone', 'identity',
            client_class=client_class.Client,
            deprecated=False,
            endpoint=self.cloud_config.config[
                'identity_endpoint_override'],
            endpoint_override=self.cloud_config.config[
                'identity_endpoint_override'])

    # Set the ironic API microversion to a known-good
    # supported/tested with the contents of shade.
    #
    # NOTE(TheJulia): Defaulted to version 1.6 as the ironic
    # state machine changes which will increment the version
    # and break an automatic transition of an enrolled node
    # to an available state. Locking the version is intended
    # to utilize the original transition until shade supports
    # calling for node inspection to allow the transition to
    # take place automatically.
    # NOTE(mordred): shade will handle microversions more
    # directly in the REST layer. This microversion property
    # will never change. When we implement REST, we should
    # start at 1.6 since that's what we've been requesting
    # via ironic_client
    @property
    def ironic_api_microversion(self):
        # NOTE(mordred) Abuse _legacy_clients to only show
        # this warning once
        if 'ironic-microversion' not in self._legacy_clients:
            warnings.warn(
                'shade is transitioning to direct REST calls which'
                ' will handle microversions with no action needed'
                ' on the part of the user. The ironic_api_microversion'
                ' property is only used by the legacy ironic_client'
                ' constructor and will never change. If you are using'
                ' it for any reason, either switch to just using'
                ' shade ironic-related API calls, or use os-client-config'
                ' make_legacy_client directly and pass os_ironic_api_version'
                ' to it as an argument. It is highly recommended to'
                ' stop using this property.')
            self._legacy_clients['ironic-microversion'] = True
        return self._get_legacy_ironic_microversion()

    def _get_legacy_ironic_microversion(self):
        return '1.6'

    @property
    def ironic_client(self):
        return self._create_legacy_client(
            'ironic', 'baremetal', deprecated=False,
            module_name='ironicclient.client.Client',
            os_ironic_api_version=self._get_legacy_ironic_microversion())

    def _get_swift_kwargs(self):
        auth_version = self.cloud_config.get_api_version('identity')
        auth_args = self.cloud_config.config.get('auth', {})
        os_options = {'auth_version': auth_version}
        if auth_version == '2.0':
            os_options['os_tenant_name'] = auth_args.get('project_name')
            os_options['os_tenant_id'] = auth_args.get('project_id')
        else:
            os_options['os_project_name'] = auth_args.get('project_name')
            os_options['os_project_id'] = auth_args.get('project_id')

        for key in (
                'username',
                'password',
                'auth_url',
                'user_id',
                'project_domain_id',
                'project_domain_name',
                'user_domain_id',
                'user_domain_name'):
            os_options['os_{key}'.format(key=key)] = auth_args.get(key)
        return os_options

    @property
    def swift_service(self):
        suppress_warning = 'swift-service' not in self._legacy_clients
        return self.make_swift_service_object(suppress_warning)

    def make_swift_service(self, suppress_warning=False):
        # NOTE(mordred): Not using helper functions because the
        #                error message needs to be different
        if not suppress_warning:
            warnings.warn(
                'Using shade to get a SwiftService object is deprecated. shade'
                ' will automatically do the things SwiftServices does as part'
                ' of the normal object resource calls. If you are having'
                ' trouble using those such that you still need to use'
                ' SwiftService, please file a bug with shade.'
                ' If you understand the issues and want to make this warning'
                ' go away, use cloud.make_swift_service(True) instead of'
                ' cloud.swift_service')
            # Abuse self._legacy_clients so that we only give the warning
            # once. We don't cache SwiftService objects.
            self._legacy_clients['swift-service'] = True
        try:
            import swiftclient.service
        except ImportError:
            self.log.error(
                'swiftclient is no longer a dependency of shade. You need to'
                ' install python-swiftclient directly.')
        with _utils.shade_exceptions("Error constructing SwiftService"):
            endpoint = self.get_session_endpoint(
                service_key='object-store')
            options = dict(os_auth_token=self.auth_token,
                           os_storage_url=endpoint,
                           os_region_name=self.region_name)
            options.update(self._get_swift_kwargs())
            return swiftclient.service.SwiftService(options=options)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
test_delete_server
----------------------------------

Tests for the `delete_server` command.
"""

from shade import exc as shade_exc
from shade.tests import fakes
from shade.tests.unit import base


class TestDeleteServer(base.RequestsMockTestCase):

    def test_delete_server(self):
        """
        Test that server delete is called when wait=False
        """
        server = fakes.make_fake_server('1234', 'daffy', 'ACTIVE')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
        ])
        self.assertTrue(self.cloud.delete_server('daffy', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone(self):
        """
        Test that we return immediately when server is already gone
        """
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': []}),
        ])
        self.assertFalse(self.cloud.delete_server('tweety', wait=False))

        self.assert_calls()

    def test_delete_server_already_gone_wait(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': []}),
        ])
        self.assertFalse(self.cloud.delete_server('speedy', wait=True))
        self.assert_calls()

    def test_delete_server_wait_for_deleted(self):
        """
        Test that delete_server waits for the server to be gone
        """
        server = fakes.make_fake_server('9999', 'wily', 'ACTIVE')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '9999'])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': []}),
        ])
        self.assertTrue(self.cloud.delete_server('wily', wait=True))

        self.assert_calls()

    def test_delete_server_fails(self):
        """
        Test that delete_server raises non-404 exceptions
        """
        server = fakes.make_fake_server('1212', 'speedy', 'ACTIVE')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1212']),
                 status_code=400),
        ])

        self.assertRaises(
            shade_exc.OpenStackCloudException,
            self.cloud.delete_server, 'speedy',
            wait=False)

        self.assert_calls()

    def test_delete_server_no_cinder(self):
        """
        Test that deleting server works when cinder is not available
        """
        orig_has_service = self.cloud.has_service

        def fake_has_service(service_type):
            if service_type == 'volume':
                return False
            return orig_has_service(service_type)
        self.cloud.has_service = fake_has_service

        server = fakes.make_fake_server('1234', 'porky', 'ACTIVE')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', 'detail']),
                 json={'servers': [server]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['servers', '1234'])),
        ])
        self.assertTrue(self.cloud.delete_server('porky', wait=False))

        self.assert_calls()



# Copyright 2017 OVH SAS
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy

from shade import exc
from shade.tests.unit import base


class TestRouter(base.RequestsMockTestCase):

    router_name = 'goofy'
    router_id = '57076620-dcfb-42ed-8ad6-79ccb4a79ed2'
    subnet_id = '1f1696eb-7f47-47f6-835c-4889bff88604'

    mock_router_rep = {
        'admin_state_up': True,
        'availability_zone_hints': [],
        'availability_zones': [],
        'description': u'',
        'distributed': False,
        'external_gateway_info': None,
        'flavor_id': None,
        'ha': False,
        'id': router_id,
        'name': router_name,
        'project_id': u'861808a93da0484ea1767967c4df8a23',
        'routes': [],
        'status': u'ACTIVE',
        'tenant_id': u'861808a93da0484ea1767967c4df8a23'
    }

    mock_router_interface_rep = {
        'network_id': '53aee281-b06d-47fc-9e1a-37f045182b8e',
        'subnet_id': '1f1696eb-7f47-47f6-835c-4889bff88604',
        'tenant_id': '861808a93da0484ea1767967c4df8a23',
        'subnet_ids': [subnet_id],
        'port_id': '23999891-78b3-4a6b-818d-d1b713f67848',
        'id': '57076620-dcfb-42ed-8ad6-79ccb4a79ed2',
        'request_ids': ['req-f1b0b1b4-ae51-4ef9-b371-0cc3c3402cf7']
    }

    def test_get_router(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': [self.mock_router_rep]})
        ])
        r = self.cloud.get_router(self.router_name)
        self.assertIsNotNone(r)
        self.assertDictEqual(self.mock_router_rep, r)
        self.assert_calls()

    def test_get_router_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': []})
        ])
        r = self.cloud.get_router('mickey')
        self.assertIsNone(r)
        self.assert_calls()

    def test_create_router(self):
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'router': self.mock_router_rep},
                 validate=dict(
                     json={'router': {
                         'name': self.router_name,
                         'admin_state_up': True}}))
        ])
        new_router = self.cloud.create_router(name=self.router_name,
                                              admin_state_up=True)
        self.assertDictEqual(self.mock_router_rep, new_router)
        self.assert_calls()

    def test_create_router_specific_tenant(self):
        new_router_tenant_id = "project_id_value"
        mock_router_rep = copy.copy(self.mock_router_rep)
        mock_router_rep['tenant_id'] = new_router_tenant_id
        mock_router_rep['project_id'] = new_router_tenant_id
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'router': mock_router_rep},
                 validate=dict(
                     json={'router': {
                         'name': self.router_name,
                         'admin_state_up': True,
                         'tenant_id': new_router_tenant_id}}))
        ])

        self.cloud.create_router(self.router_name,
                                 project_id=new_router_tenant_id)
        self.assert_calls()

    def test_create_router_with_enable_snat_True(self):
        """Do not send enable_snat when same as neutron default."""
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'router': self.mock_router_rep},
                 validate=dict(
                     json={'router': {
                         'name': self.router_name,
                         'admin_state_up': True}}))
        ])
        self.cloud.create_router(
            name=self.router_name, admin_state_up=True, enable_snat=True)
        self.assert_calls()

    def test_create_router_with_enable_snat_False(self):
        """Send enable_snat when it is False."""
        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'router': self.mock_router_rep},
                 validate=dict(
                     json={'router': {
                         'name': self.router_name,
                         'external_gateway_info': {'enable_snat': False},
                         'admin_state_up': True}}))
        ])
        self.cloud.create_router(
            name=self.router_name, admin_state_up=True, enable_snat=False)
        self.assert_calls()

    def test_add_router_interface(self):
        self.register_uris([
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'routers', self.router_id,
                             'add_router_interface.json']),
                 json={'port': self.mock_router_interface_rep},
                 validate=dict(
                     json={'subnet_id': self.subnet_id}))
        ])
        self.cloud.add_router_interface(
            {'id': self.router_id}, subnet_id=self.subnet_id)
        self.assert_calls()

    def test_remove_router_interface(self):
        self.register_uris([
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'routers', self.router_id,
                             'remove_router_interface.json']),
                 json={'port': self.mock_router_interface_rep},
                 validate=dict(
                     json={'subnet_id': self.subnet_id}))
        ])
        self.cloud.remove_router_interface(
            {'id': self.router_id}, subnet_id=self.subnet_id)
        self.assert_calls()

    def test_remove_router_interface_missing_argument(self):
        self.assertRaises(ValueError, self.cloud.remove_router_interface,
                          {'id': '123'})

    def test_update_router(self):
        new_router_name = "mickey"
        expected_router_rep = copy.copy(self.mock_router_rep)
        expected_router_rep['name'] = new_router_name
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': [self.mock_router_rep]}),
            dict(method='PUT',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'routers', '%s.json' % self.router_id]),
                 json={'router': expected_router_rep},
                 validate=dict(
                     json={'router': {
                         'name': new_router_name}}))
        ])
        new_router = self.cloud.update_router(
            self.router_id, name=new_router_name)
        self.assertDictEqual(expected_router_rep, new_router)
        self.assert_calls()

    def test_delete_router(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': [self.mock_router_rep]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'routers', '%s.json' % self.router_id]),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_router(self.router_name))
        self.assert_calls()

    def test_delete_router_not_found(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': []}),
        ])
        self.assertFalse(self.cloud.delete_router(self.router_name))
        self.assert_calls()

    def test_delete_router_multiple_found(self):
        router1 = dict(id='123', name='mickey')
        router2 = dict(id='456', name='mickey')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': [router1, router2]}),
        ])
        self.assertRaises(exc.OpenStackCloudException,
                          self.cloud.delete_router,
                          'mickey')
        self.assert_calls()

    def test_delete_router_multiple_using_id(self):
        router1 = dict(id='123', name='mickey')
        router2 = dict(id='456', name='mickey')
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'routers.json']),
                 json={'routers': [router1, router2]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'network', 'public',
                     append=['v2.0', 'routers', '123.json']),
                 json={})
        ])
        self.assertTrue(self.cloud.delete_router("123"))
        self.assert_calls()

    def _test_list_router_interfaces(self, router, interface_type,
                                     expected_result=None):
        internal_port = {
            'id': 'internal_port_id',
            'fixed_ips': [{
                'subnet_id': 'internal_subnet_id',
                'ip_address': "10.0.0.1"
            }],
            'device_id': self.router_id
        }
        external_port = {
            'id': 'external_port_id',
            'fixed_ips': [{
                'subnet_id': 'external_subnet_id',
                'ip_address': "1.2.3.4"
            }],
            'device_id': self.router_id
        }
        if expected_result is None:
            if interface_type == "internal":
                expected_result = [internal_port]
            elif interface_type == "external":
                expected_result = [external_port]
            else:
                expected_result = [internal_port, external_port]

        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'network', 'public', append=['v2.0', 'ports.json'],
                     qs_elements=["device_id=%s" % self.router_id]),
                 json={'ports': [internal_port, external_port]})
        ])
        ret = self.cloud.list_router_interfaces(router, interface_type)
        self.assertEqual(expected_result, ret)
        self.assert_calls()

    def test_list_router_interfaces_no_gw(self):
        """
        If a router does not have external_gateway_info, do not fail.
        """
        router = {
            'id': self.router_id
        }
        self._test_list_router_interfaces(router,
                                          interface_type="external",
                                          expected_result=[])

    def test_list_router_interfaces_gw_none(self):
        """
        If a router does have external_gateway_info set to None, do not fail.
        """
        router = {
            'id': self.router_id,
            'external_gateway_info': None
        }
        self._test_list_router_interfaces(router,
                                          interface_type="external",
                                          expected_result=[])

    def test_list_router_interfaces_all(self):
        router = {
            'id': self.router_id,
            'external_gateway_info': {
                'external_fixed_ips': [{
                    'subnet_id': 'external_subnet_id',
                    'ip_address': '1.2.3.4'}]
            }
        }
        self._test_list_router_interfaces(router,
                                          interface_type=None)

    def test_list_router_interfaces_internal(self):
        router = {
            'id': self.router_id,
            'external_gateway_info': {
                'external_fixed_ips': [{
                    'subnet_id': 'external_subnet_id',
                    'ip_address': '1.2.3.4'}]
            }
        }
        self._test_list_router_interfaces(router,
                                          interface_type="internal")

    def test_list_router_interfaces_external(self):
        router = {
            'id': self.router_id,
            'external_gateway_info': {
                'external_fixed_ips': [{
                    'subnet_id': 'external_subnet_id',
                    'ip_address': '1.2.3.4'}]
            }
        }
        self._test_list_router_interfaces(router,
                                          interface_type="external")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from shade import meta
from shade.tests.unit import base


class TestVolumeBackups(base.RequestsMockTestCase):
    def test_search_volume_backups(self):
        name = 'Volume1'
        vol1 = {'name': name, 'availability_zone': 'az1'}
        vol2 = {'name': name, 'availability_zone': 'az1'}
        vol3 = {'name': 'Volume2', 'availability_zone': 'az2'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public', append=['backups', 'detail']),
                 json={"backups": [vol1, vol2, vol3]})])
        result = self.cloud.search_volume_backups(
            name, {'availability_zone': 'az1'})
        self.assertEqual(len(result), 2)
        self.assertEqual(
            meta.obj_list_to_munch([vol1, vol2]),
            result)
        self.assert_calls()

    def test_get_volume_backup(self):
        name = 'Volume1'
        vol1 = {'name': name, 'availability_zone': 'az1'}
        vol2 = {'name': name, 'availability_zone': 'az2'}
        vol3 = {'name': 'Volume2', 'availability_zone': 'az1'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public', append=['backups', 'detail']),
                 json={"backups": [vol1, vol2, vol3]})])
        result = self.cloud.get_volume_backup(
            name, {'availability_zone': 'az1'})
        result = meta.obj_to_munch(result)
        self.assertEqual(
            meta.obj_to_munch(vol1),
            result)
        self.assert_calls()

    def test_list_volume_backups(self):
        backup = {'id': '6ff16bdf-44d5-4bf9-b0f3-687549c76414',
                  'status': 'available'}
        search_opts = {'status': 'available'}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public', append=['backups', 'detail'],
                     qs_elements=['='.join(i) for i in search_opts.items()]),
                 json={"backups": [backup]})])
        result = self.cloud.list_volume_backups(True, search_opts)
        self.assertEqual(len(result), 1)
        self.assertEqual(
            meta.obj_list_to_munch([backup]),
            result)
        self.assert_calls()

    def test_delete_volume_backup_wait(self):
        backup_id = '6ff16bdf-44d5-4bf9-b0f3-687549c76414'
        backup = {'id': backup_id}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', backup_id])),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": []})])
        self.cloud.delete_volume_backup(backup_id, False, True, 1)
        self.assert_calls()

    def test_delete_volume_backup_force(self):
        backup_id = '6ff16bdf-44d5-4bf9-b0f3-687549c76414'
        backup = {'id': backup_id}
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='POST',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', backup_id, 'action']),
                 json={'os-force_delete': {}},
                 validate=dict(json={u'os-force_delete': None})),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": [backup]}),
            dict(method='GET',
                 uri=self.get_mock_url(
                     'volumev2', 'public',
                     append=['backups', 'detail']),
                 json={"backups": []})
        ])
        self.cloud.delete_volume_backup(backup_id, True, True, 1)
        self.assert_calls()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import tempfile

import testtools

import shade
import shade.openstackcloud
from shade import exc
from shade.tests.unit import base


class BaseTestObject(base.RequestsMockTestCase):

    def setUp(self):
        super(BaseTestObject, self).setUp()

        self.container = self.getUniqueString()
        self.object = self.getUniqueString()
        self.endpoint = self.cloud._object_store_client.get_endpoint()
        self.container_endpoint = '{endpoint}/{container}'.format(
            endpoint=self.endpoint, container=self.container)
        self.object_endpoint = '{endpoint}/{object}'.format(
            endpoint=self.container_endpoint, object=self.object)


class TestObject(BaseTestObject):

    def test_create_container(self):
        """Test creating a (private) container"""
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint, status_code=404),
            dict(method='PUT', uri=self.container_endpoint,
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD', uri=self.container_endpoint,
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'})
        ])

        self.cloud.create_container(self.container)
        self.assert_calls()

    def test_create_container_public(self):
        """Test creating a public container"""
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint,
                 status_code=404),
            dict(method='PUT', uri=self.container_endpoint,
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='POST', uri=self.container_endpoint,
                 status_code=201,
                 validate=dict(
                     headers={
                         'x-container-read':
                             shade.openstackcloud.OBJECT_CONTAINER_ACLS[
                                 'public']})),
            dict(method='HEAD', uri=self.container_endpoint,
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'})
        ])

        self.cloud.create_container(self.container, public=True)
        self.assert_calls()

    def test_create_container_exists(self):
        """Test creating a container that exists."""
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint,
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'})
        ])

        container = self.cloud.create_container(self.container)

        self.assert_calls()
        self.assertIsNotNone(container)

    def test_delete_container(self):
        self.register_uris([
            dict(method='DELETE', uri=self.container_endpoint)])

        self.assertTrue(self.cloud.delete_container(self.container))
        self.assert_calls()

    def test_delete_container_404(self):
        """No exception when deleting a container that does not exist"""
        self.register_uris([
            dict(method='DELETE', uri=self.container_endpoint,
                 status_code=404)])

        self.assertFalse(self.cloud.delete_container(self.container))
        self.assert_calls()

    def test_delete_container_error(self):
        """Non-404 swift error re-raised as OSCE"""
        # 409 happens if the container is not empty
        self.register_uris([
            dict(method='DELETE', uri=self.container_endpoint,
                 status_code=409)])
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.delete_container, self.container)
        self.assert_calls()

    def test_update_container(self):
        headers = {
            'x-container-read':
            shade.openstackcloud.OBJECT_CONTAINER_ACLS['public']}
        self.register_uris([
            dict(method='POST', uri=self.container_endpoint,
                 status_code=204,
                 validate=dict(headers=headers))])

        self.cloud.update_container(self.container, headers)
        self.assert_calls()

    def test_update_container_error(self):
        """Swift error re-raised as OSCE"""
        # This test is of questionable value - the swift API docs do not
        # declare error codes (other than 404 for the container) for this
        # method, and I cannot make a synthetic failure to validate a real
        # error code. So we're really just testing the shade adapter error
        # raising logic here, rather than anything specific to swift.
        self.register_uris([
            dict(method='POST', uri=self.container_endpoint,
                 status_code=409)])
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.update_container, self.container, dict(foo='bar'))
        self.assert_calls()

    def test_set_container_access_public(self):
        self.register_uris([
            dict(method='POST', uri=self.container_endpoint,
                 status_code=204,
                 validate=dict(
                     headers={
                         'x-container-read':
                             shade.openstackcloud.OBJECT_CONTAINER_ACLS[
                                 'public']}))])

        self.cloud.set_container_access(self.container, 'public')

        self.assert_calls()

    def test_set_container_access_private(self):
        self.register_uris([
            dict(method='POST', uri=self.container_endpoint,
                 status_code=204,
                 validate=dict(
                     headers={
                         'x-container-read':
                             shade.openstackcloud.OBJECT_CONTAINER_ACLS[
                                 'private']}))])

        self.cloud.set_container_access(self.container, 'private')

        self.assert_calls()

    def test_set_container_access_invalid(self):
        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.set_container_access, self.container, 'invalid')

    def test_get_container_access(self):
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint,
                 headers={
                     'x-container-read':
                         str(shade.openstackcloud.OBJECT_CONTAINER_ACLS[
                             'public'])})])
        access = self.cloud.get_container_access(self.container)
        self.assertEqual('public', access)

    def test_get_container_invalid(self):
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint,
                 headers={'x-container-read': 'invalid'})])

        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Could not determine container access for ACL: invalid"
        ):
            self.cloud.get_container_access(self.container)

    def test_get_container_access_not_found(self):
        self.register_uris([
            dict(method='HEAD', uri=self.container_endpoint,
                 status_code=404)])
        with testtools.ExpectedException(
                exc.OpenStackCloudException,
                "Container not found: %s" % self.container
        ):
            self.cloud.get_container_access(self.container)

    def test_list_containers(self):
        endpoint = '{endpoint}/?format=json'.format(
            endpoint=self.endpoint)
        containers = [
            {u'count': 0, u'bytes': 0, u'name': self.container}]

        self.register_uris([dict(method='GET', uri=endpoint, complete_qs=True,
                                 json=containers)])

        ret = self.cloud.list_containers()

        self.assert_calls()
        self.assertEqual(containers, ret)

    def test_list_containers_exception(self):
        endpoint = '{endpoint}/?format=json'.format(
            endpoint=self.endpoint)
        self.register_uris([dict(method='GET', uri=endpoint, complete_qs=True,
                                 status_code=416)])

        self.assertRaises(
            exc.OpenStackCloudException, self.cloud.list_containers)
        self.assert_calls()

    def test_list_objects(self):
        endpoint = '{endpoint}?format=json'.format(
            endpoint=self.container_endpoint)

        objects = [{
            u'bytes': 20304400896,
            u'last_modified': u'2016-12-15T13:34:13.650090',
            u'hash': u'daaf9ed2106d09bba96cf193d866445e',
            u'name': self.object,
            u'content_type': u'application/octet-stream'}]

        self.register_uris([dict(method='GET', uri=endpoint, complete_qs=True,
                                 json=objects)])

        ret = self.cloud.list_objects(self.container)

        self.assert_calls()
        self.assertEqual(objects, ret)

    def test_list_objects_exception(self):
        endpoint = '{endpoint}?format=json'.format(
            endpoint=self.container_endpoint)
        self.register_uris([dict(method='GET', uri=endpoint, complete_qs=True,
                                 status_code=416)])
        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.list_objects, self.container)
        self.assert_calls()

    def test_delete_object(self):
        self.register_uris([
            dict(method='HEAD', uri=self.object_endpoint,
                 headers={'X-Object-Meta': 'foo'}),
            dict(method='DELETE', uri=self.object_endpoint, status_code=204)])

        self.assertTrue(self.cloud.delete_object(self.container, self.object))

        self.assert_calls()

    def test_delete_object_not_found(self):
        self.register_uris([dict(method='HEAD', uri=self.object_endpoint,
                                 status_code=404)])

        self.assertFalse(self.cloud.delete_object(self.container, self.object))

        self.assert_calls()

    def test_get_object(self):
        headers = {
            'Content-Length': '20304400896',
            'Content-Type': 'application/octet-stream',
            'Accept-Ranges': 'bytes',
            'Last-Modified': 'Thu, 15 Dec 2016 13:34:14 GMT',
            'Etag': '"b5c454b44fbd5344793e3fb7e3850768"',
            'X-Timestamp': '1481808853.65009',
            'X-Trans-Id': 'tx68c2a2278f0c469bb6de1-005857ed80dfw1',
            'Date': 'Mon, 19 Dec 2016 14:24:00 GMT',
            'X-Static-Large-Object': 'True',
            'X-Object-Meta-Mtime': '1481513709.168512',
        }
        response_headers = {k.lower(): v for k, v in headers.items()}
        text = 'test body'
        self.register_uris([
            dict(method='GET', uri=self.object_endpoint,
                 headers={
                     'Content-Length': '20304400896',
                     'Content-Type': 'application/octet-stream',
                     'Accept-Ranges': 'bytes',
                     'Last-Modified': 'Thu, 15 Dec 2016 13:34:14 GMT',
                     'Etag': '"b5c454b44fbd5344793e3fb7e3850768"',
                     'X-Timestamp': '1481808853.65009',
                     'X-Trans-Id': 'tx68c2a2278f0c469bb6de1-005857ed80dfw1',
                     'Date': 'Mon, 19 Dec 2016 14:24:00 GMT',
                     'X-Static-Large-Object': 'True',
                     'X-Object-Meta-Mtime': '1481513709.168512',
                 },
                 text='test body')])

        resp = self.cloud.get_object(self.container, self.object)

        self.assert_calls()

        self.assertEqual((response_headers, text), resp)

    def test_get_object_not_found(self):
        self.register_uris([dict(method='GET',
                                 uri=self.object_endpoint, status_code=404)])

        self.assertIsNone(self.cloud.get_object(self.container, self.object))

        self.assert_calls()

    def test_get_object_exception(self):
        self.register_uris([dict(method='GET', uri=self.object_endpoint,
                                 status_code=416)])

        self.assertRaises(
            shade.OpenStackCloudException,
            self.cloud.get_object,
            self.container, self.object)

        self.assert_calls()

    def test_get_object_segment_size_below_min(self):
        # Register directly becuase we make multiple calls. The number
        # of calls we make isn't interesting - what we do with the return
        # values is. Don't run assert_calls for the same reason.
        self.register_uris([
            dict(method='GET', uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': 1000},
                     slo={'min_segment_size': 500}),
                 headers={'Content-Type': 'application/json'})])
        self.assertEqual(500, self.cloud.get_object_segment_size(400))
        self.assertEqual(900, self.cloud.get_object_segment_size(900))
        self.assertEqual(1000, self.cloud.get_object_segment_size(1000))
        self.assertEqual(1000, self.cloud.get_object_segment_size(1100))

    def test_get_object_segment_size_http_404(self):
        self.register_uris([
            dict(method='GET', uri='https://object-store.example.com/info',
                 status_code=404, reason='Not Found')])
        self.assertEqual(shade.openstackcloud.DEFAULT_OBJECT_SEGMENT_SIZE,
                         self.cloud.get_object_segment_size(None))
        self.assert_calls()

    def test_get_object_segment_size_http_412(self):
        self.register_uris([
            dict(method='GET', uri='https://object-store.example.com/info',
                 status_code=412, reason='Precondition failed')])
        self.assertEqual(shade.openstackcloud.DEFAULT_OBJECT_SEGMENT_SIZE,
                         self.cloud.get_object_segment_size(None))
        self.assert_calls()


class TestObjectUploads(BaseTestObject):

    def setUp(self):
        super(TestObjectUploads, self).setUp()

        self.content = self.getUniqueString().encode('latin-1')
        self.object_file = tempfile.NamedTemporaryFile(delete=False)
        self.object_file.write(self.content)
        self.object_file.close()
        (self.md5, self.sha256) = self.cloud._get_file_hashes(
            self.object_file.name)
        self.endpoint = self.cloud._object_store_client.get_endpoint()

    def test_create_object(self):

        self.register_uris([
            dict(method='GET',
                 uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': 1000},
                     slo={'min_segment_size': 500})),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint, container=self.container),
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint, container=self.container),
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'}),
            dict(method='HEAD',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint, container=self.container,
                     object=self.object),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=201,
                 validate=dict(
                     headers={
                         'x-object-meta-x-shade-md5': self.md5,
                         'x-object-meta-x-shade-sha256': self.sha256,
                     }))
        ])

        self.cloud.create_object(
            container=self.container, name=self.object,
            filename=self.object_file.name)

        self.assert_calls()

    def test_create_dynamic_large_object(self):

        max_file_size = 2
        min_file_size = 1

        uris_to_mock = [
            dict(method='GET',
                 uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': max_file_size},
                     slo={'min_segment_size': min_file_size})),

            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container, ),
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'}),
            dict(method='HEAD',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint, container=self.container,
                     object=self.object),
                 status_code=404)
        ]

        uris_to_mock.extend(
            [dict(method='PUT',
                  uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
                      endpoint=self.endpoint,
                      container=self.container,
                      object=self.object,
                      index=index),
                  status_code=201)
                for index, offset in enumerate(
                    range(0, len(self.content), max_file_size))]
        )

        uris_to_mock.append(
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=201,
                 validate=dict(
                     headers={
                         'x-object-manifest': '{container}/{object}'.format(
                             container=self.container, object=self.object),
                         'x-object-meta-x-shade-md5': self.md5,
                         'x-object-meta-x-shade-sha256': self.sha256,
                     })))
        self.register_uris(uris_to_mock)
        self.cloud.create_object(
            container=self.container, name=self.object,
            filename=self.object_file.name, use_slo=False)

        # After call 6, order become indeterminate because of thread pool
        self.assert_calls(stop_after=6)

        for key, value in self.calls[-1]['headers'].items():
            self.assertEqual(
                value, self.adapter.request_history[-1].headers[key],
                'header mismatch in manifest call')

    def test_create_static_large_object(self):

        max_file_size = 25
        min_file_size = 1

        uris_to_mock = [
            dict(method='GET', uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': max_file_size},
                     slo={'min_segment_size': min_file_size})),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container, ),
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'}),
            dict(method='HEAD',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=404)
        ]

        uris_to_mock.extend([
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/{index:0>6}'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object,
                     index=index),
                 status_code=201,
                 headers=dict(Etag='etag{index}'.format(index=index)))
            for index, offset in enumerate(
                range(0, len(self.content), max_file_size))
        ])

        uris_to_mock.append(
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=201,
                 validate=dict(
                     params={
                         'multipart-manifest', 'put'
                     },
                     headers={
                         'x-object-meta-x-shade-md5': self.md5,
                         'x-object-meta-x-shade-sha256': self.sha256,
                     })))
        self.register_uris(uris_to_mock)

        self.cloud.create_object(
            container=self.container, name=self.object,
            filename=self.object_file.name, use_slo=True)

        # After call 6, order become indeterminate because of thread pool
        self.assert_calls(stop_after=6)

        for key, value in self.calls[-1]['headers'].items():
            self.assertEqual(
                value, self.adapter.request_history[-1].headers[key],
                'header mismatch in manifest call')

        base_object = '/{container}/{object}'.format(
            endpoint=self.endpoint,
            container=self.container,
            object=self.object)

        self.assertEqual([
            {
                'path': "{base_object}/000000".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag0',
            },
            {
                'path': "{base_object}/000001".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag1',
            },
            {
                'path': "{base_object}/000002".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag2',
            },
            {
                'path': "{base_object}/000003".format(
                    base_object=base_object),
                'size_bytes': 5,
                'etag': 'etag3',
            },
        ], self.adapter.request_history[-1].json())

    def test_object_segment_retry_failure(self):

        max_file_size = 25
        min_file_size = 1

        self.register_uris([
            dict(method='GET', uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': max_file_size},
                     slo={'min_segment_size': min_file_size})),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container, ),
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'}),
            dict(method='HEAD',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000000'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000001'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000002'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000003'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=501),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=201)
        ])

        self.assertRaises(
            exc.OpenStackCloudException,
            self.cloud.create_object,
            container=self.container, name=self.object,
            filename=self.object_file.name, use_slo=True)

        # After call 6, order become indeterminate because of thread pool
        self.assert_calls(stop_after=6)

    def test_object_segment_retries(self):

        max_file_size = 25
        min_file_size = 1

        self.register_uris([
            dict(method='GET', uri='https://object-store.example.com/info',
                 json=dict(
                     swift={'max_file_size': max_file_size},
                     slo={'min_segment_size': min_file_size})),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container, ),
                 status_code=201,
                 headers={
                     'Date': 'Fri, 16 Dec 2016 18:21:20 GMT',
                     'Content-Length': '0',
                     'Content-Type': 'text/html; charset=UTF-8',
                 }),
            dict(method='HEAD',
                 uri='{endpoint}/{container}'.format(
                     endpoint=self.endpoint,
                     container=self.container),
                 headers={
                     'Content-Length': '0',
                     'X-Container-Object-Count': '0',
                     'Accept-Ranges': 'bytes',
                     'X-Storage-Policy': 'Policy-0',
                     'Date': 'Fri, 16 Dec 2016 18:29:05 GMT',
                     'X-Timestamp': '1481912480.41664',
                     'X-Trans-Id': 'tx60ec128d9dbf44b9add68-0058543271dfw1',
                     'X-Container-Bytes-Used': '0',
                     'Content-Type': 'text/plain; charset=utf-8'}),
            dict(method='HEAD',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=404),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000000'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 headers={'etag': 'etag0'},
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000001'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 headers={'etag': 'etag1'},
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000002'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 headers={'etag': 'etag2'},
                 status_code=201),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000003'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=501),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}/000003'.format(
                     endpoint=self.endpoint,
                     container=self.container,
                     object=self.object),
                 status_code=201,
                 headers={'etag': 'etag3'}),
            dict(method='PUT',
                 uri='{endpoint}/{container}/{object}'.format(
                     endpoint=self.endpoint,
                     container=self.container, object=self.object),
                 status_code=201,
                 validate=dict(
                     params={
                         'multipart-manifest', 'put'
                     },
                     headers={
                         'x-object-meta-x-shade-md5': self.md5,
                         'x-object-meta-x-shade-sha256': self.sha256,
                     }))
        ])

        self.cloud.create_object(
            container=self.container, name=self.object,
            filename=self.object_file.name, use_slo=True)

        # After call 6, order become indeterminate because of thread pool
        self.assert_calls(stop_after=6)

        for key, value in self.calls[-1]['headers'].items():
            self.assertEqual(
                value, self.adapter.request_history[-1].headers[key],
                'header mismatch in manifest call')

        base_object = '/{container}/{object}'.format(
            endpoint=self.endpoint,
            container=self.container,
            object=self.object)

        self.assertEqual([
            {
                'path': "{base_object}/000000".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag0',
            },
            {
                'path': "{base_object}/000001".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag1',
            },
            {
                'path': "{base_object}/000002".format(
                    base_object=base_object),
                'size_bytes': 25,
                'etag': 'etag2',
            },
            {
                'path': "{base_object}/000003".format(
                    base_object=base_object),
                'size_bytes': 1,
                'etag': 'etag3',
            },
        ], self.adapter.request_history[-1].json())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import uuid

from shade.tests.unit import base
from shade.tests import fakes


class TestServerGroup(base.RequestsMockTestCase):

    def setUp(self):
        super(TestServerGroup, self).setUp()
        self.group_id = uuid.uuid4().hex
        self.group_name = self.getUniqueString('server-group')
        self.policies = ['affinity']
        self.fake_group = fakes.make_fake_server_group(
            self.group_id, self.group_name, self.policies)

    def test_create_server_group(self):

        self.register_uris([
            dict(method='POST',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-server-groups']),
                 json={'server_group': self.fake_group},
                 validate=dict(
                     json={'server_group': {
                         'name': self.group_name,
                         'policies': self.policies,
                     }})),
        ])

        self.cloud.create_server_group(name=self.group_name,
                                       policies=self.policies)

        self.assert_calls()

    def test_delete_server_group(self):
        self.register_uris([
            dict(method='GET',
                 uri=self.get_mock_url(
                     'compute', 'public', append=['os-server-groups']),
                 json={'server_groups': [self.fake_group]}),
            dict(method='DELETE',
                 uri=self.get_mock_url(
                     'compute', 'public',
                     append=['os-server-groups', self.group_id]),
                 json={'server_groups': [self.fake_group]}),
        ])
        self.assertTrue(self.cloud.delete_server_group(self.group_name))

        self.assert_calls()



import shade
shade.simple_logging(debug=True)

cloud = shade.openstack_cloud(cloud='kiss', region_name='region1')
print(cloud.has_service('network'))
print(cloud.has_service('container-orchestration'))
